@@ -92,17 +92,28 @@ One or more stream URLs can be returned for each station.
 
 =over 4
 
-=item B<new>(I<url> [, I<streamtype> | "debug" [ => 0|1|2 ] ... ])
+=item B<new>(I<url> [, I<-keep|-skip> => I<streamtypes> | I<-debug> [ => 0|1|2 ] ... ])
 
 Accepts an iheartradio.com URL and creates and returns a new station object, or 
 I<undef> if the URL is not a valid iheartradio station or no streams are found.
 
-The optional I<streamtype> can be one of:  any, secure, secure_pls, pls, 
-secure_hls, hls, secure_shortcast, shortcast, secure_rtmp, rtmp, etc.  More 
-than one value can be specified to control order of search.  A I<streamtype> 
-can be preceeded by an exclamantion point ("!") to reject that type of stream.
-If "any" appears in the list, it should be the last specifier without a "!" 
-preceeding it, and itself should not be preceeded with a "!" (inverter)!  
+I<-keep> and I<-skip> specify a list of one or more I<streamtypes> to either 
+include or skip respectively.  The list for each can be either a comma-separated 
+string or an array reference ([...]) of stream types, in the order they should be 
+returned.  Each stream type in the list can be one of:  any, secure, secure_pls, 
+pls, secure_hls, hls, secure_shortcast, shortcast, secure_rtmp, rtmp, etc.  
+NOTE:  This is now the preferred method over the DEPRECIATED one below.  If using 
+this method, do NOT include the inverter ("!") in front of the stream types, as 
+this is not used - these should be put in the I<-skip> list now.  The method 
+below will be REMOVED in a later version soon!
+
+DEPRECIATED:  use The optional I<streamtype> can be one of:  any, secure, 
+secure_pls, pls, secure_hls, hls, secure_shortcast, shortcast, secure_rtmp, 
+rtmp, etc.  More than one value can be specified to control order of search.  
+A I<streamtype> can be preceeded by an exclamantion point ("!") to reject that 
+type of stream.  If "any" appears in the list, it should be the last specifier 
+without a "!" preceeding it, and itself should not be preceeded with a "!" 
+(inverter)!  
 
 For example, the list:  'secure_shoutcast', 'secure', 'any', '!rtmp' 
 would try to find a "secure_shoutcast" (https) shortcast stream, if none found, 
@@ -114,11 +125,18 @@ streams.
 
 Returns an array of strings representing all stream urls found.
 
-=item $station->B<getURL>()
+=item $station->B<getURL>([I<options>])
 
 Similar to B<get>() except it only returns a single stream representing 
 the first valid stream found.  
 
+Current options are:  I<-random> and I<-noplaylists>.  By default, the 
+first ("best"?) stream is returned.  If I<-random> is specified, then 
+a random one is selected from the list of streams found.  
+If I<-noplaylists> is specified, and the stream to be returned is a 
+"playlist" (.pls extension), it is first fetched and the first entry 
+in the playlist is returned.  This is needed by Fauxdacious Mediaplayer.
+
 =item $station->B<count>()
 
 Returns the number of streams found for the station.
@@ -156,6 +174,44 @@ Returns the stream's type ("IHeartRadio").
 
 =back
 
+=head1 CONFIGURATION FILES
+
+=over 4
+
+=item ~/.config/StreamFinder/IHeartRadio/config
+
+Optional text file for specifying various configuration options 
+for a specific site (submodule).  Each option is specified on a 
+separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used only by the specific 
+(submodule) specified.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.  
+Blank lines and lines starting with a "#" sign are ignored.
+
+Options specified here override any specified in I<~/.config/StreamFinder/config>.
+
+Among options valid for IHeartRadio streams are the I<-keep> and 
+I<-skip> options described in the B<new()> function.
+
+=item ~/.config/StreamFinder/config
+
+Optional text file for specifying various configuration options.  
+Each option is specified on a separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used by all sites 
+(submodules) that support them.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.
+
+=back
+
+NOTE:  Options specified in the options parameter list will override 
+those corresponding options specified in these files.
+
 =head1 KEYWORDS
 
 iheartradio
@@ -247,7 +303,9 @@ use warnings;
 use LWP::UserAgent ();
 use vars qw(@ISA @EXPORT);
 
-our $DEBUG = 0;
+my $DEBUG = 0;
+my %uops = ();
+my @userAgentOps = ();
 
 require Exporter;
 
@@ -258,33 +316,75 @@ sub new
 {
 	my $class = shift;
 	my $url = shift;
-	my (@okStreams, @skipStreams);
+
+	my $self = {};
+	return undef  unless ($url);
+
+	my (@okStreams, @skipStreams, @okStreamsClassic, @skipStreamsClassic);
+	foreach my $p ("$ENV{HOME}/.config/StreamFinder/config", "$ENV{HOME}/.config/StreamFinder/IHeartRadio/config") {
+		if (open IN, $p) {
+			my ($atr, $val);
+			while (<IN>) {
+				chomp;
+				next  if (/^\s*\#/o);
+				($atr, $val) = split(/\s*\=\>\s*/o, $_, 2);
+				eval "\$uops{$atr} = $val";
+			}
+			close IN;
+		}
+	}
+	foreach my $i (qw(agent from conn_cache default_headers local_address ssl_opts max_size
+			max_redirect parse_head protocols_allowed protocols_forbidden requests_redirectable
+			proxy no_proxy)) {
+		push @userAgentOps, $i, $uops{$i}  if (defined $uops{$i});
+	}
+	push (@userAgentOps, 'agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0')
+			unless (defined $uops{'agent'});
+	$uops{'timeout'} = 10  unless (defined $uops{'timeout'});
+	$DEBUG = $uops{'debug'}  if (defined $uops{'debug'});
+
 	while (@_) {
-		if ($_[0] =~ /^\!/o) {
+		if ($_[0] =~ /^\!/o) {  #DEPRECIATED, USE -keep AND/OR -skip!
 			(my $i = shift) =~ s/\!//o;
-			push @skipStreams, $i;
+			push @skipStreamsClassic, $i;
 		} elsif ($_[0] =~ /^\-?debug$/o) {
 			shift;
 			$DEBUG = (defined($_[0]) && $_[0] =~/^[0-9]$/) ? shift : 1;
-print STDERR "-???- DEBUG=$DEBUG=\n";
-		} else {
-			push @okStreams, shift;
+		} elsif ($_[0] =~ /^\-?keep$/o) {
+			shift;
+			if (defined $_[0]) {
+				my $keeporder = shift;
+				@okStreams = (ref($keeporder) =~ /ARRAY/) ? @{$keeporder} : split(/\,\s*/, $keeporder);
+			}
+		} elsif ($_[0] =~ /^\-?skip$/o) {
+			shift;
+			if (defined $_[0]) {
+				my $skiporder = shift;
+				@skipStreams = (ref($skiporder) =~ /ARRAY/) ? @{$skiporder} : split(/\,\s*/, $skiporder);
+			}
+		} else {  #DEPRECIATED, USE -keep AND/OR -skip!
+			push @okStreamsClassic, shift;
 		}
-	}	
+	}
+	@okStreams = @okStreamsClassic  unless (defined $okStreams[0]);
+	@skipStreams = @skipStreamsClassic  unless (defined $skipStreams[0]);
+	if (!defined($okStreams[0]) && defined($uops{'keep'})) {
+		@okStreams = (ref($uops{'keep'}) =~ /ARRAY/) ? @{$uops{'keep'}} : split(/\,\s*/, $uops{'keep'});
+	}
+	if (!defined($skipStreams[0]) && defined($uops{'skip'})) {
+		@skipStreams = (ref($uops{'skip'}) =~ /ARRAY/) ? @{$uops{'skip'}} : split(/\,\s*/, $uops{'skip'});
+	}
 	@okStreams = ('any')  unless (defined $okStreams[0]);  # one of:  {secure_pls | pls | stw}
 
-	my $self = {};
-
-	print STDERR "-0(IHeartRadio): URL=$url=\n"  if ($DEBUG);
-	return undef  unless ($url);
+	print STDERR "-0(IHeartRadio): URL=$url= KEEP=(",join('|',@okStreams).") SKIP=(",join('|',@skipStreams).")\n"  if ($DEBUG);
 
 	(my $url2fetch = $url);
 	$url2fetch = 'https://www.iheart.com/live/' . $url . '/'  unless ($url =~ /^http/);
 	$self->{'id'} = $1  if ($url2fetch =~ m#\/([^\/]+)\/$#);
 	my $html = '';
 	print STDERR "-0(IHeartRadio): URL=$url2fetch=\n"  if ($DEBUG);
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $response = $ua->get($url2fetch);
@@ -299,7 +399,27 @@ print STDERR "-???- DEBUG=$DEBUG=\n";
 	my $html2 = '';
 	my $streamhtml0 = ($html =~ /\"streams\"\s*\:\s*\{([^\}]+)\}/) ? $1 : '';
 	print STDERR "-2: streamhtml=$streamhtml0=\n"  if ($DEBUG);
-	return undef  unless ($streamhtml0);
+	$self->{'cnt'} = 0;
+	unless ($streamhtml0) {  #NO STREAMS (PODCAST?) - LOOK FOR MEDIAURL:
+		while ($html =~ s#\"mediaUrl\"\:\"([^\"]+)\"##gso) {
+			push @{$self->{'streams'}}, $1;
+			$self->{'cnt'}++;
+		}
+		return undef  unless ($self->{'cnt'} > 0);
+		my $id = $url;
+		$id =~ s#\/$##;
+		$id = $1  if ($id =~ m#([^\/]+)$#);
+		$self->{'id'} = $id;
+		$self->{'title'} = ($html =~ s#\<title[^\>]+\>([^\<]+)\<\/title\>##s) ? $1 : '';
+		$self->{'imageurl'} = ($html =~ s#\"imageUrl\"\:\"([^\"]+)\"##s) ? $1 : '';
+		$self->{'iconurl'} = $self->{'imageurl'};
+		$self->{'total'} = $self->{'cnt'};
+		print STDERR "-SUCCESS2: --id=".$self->{'id'}."=\n--tit=".$self->{'title'}."=\n--icon=".$self->{'iconurl'}."=\n--streams=(".join('|',@{$self->{'streams'}}).")\n"  if ($DEBUG);
+
+		bless $self, $class;   #BLESS IT!
+
+		return $self;
+	}
 
 	my ($streamhtml, $streampattern);
 	my %streams = ();
@@ -307,7 +427,7 @@ print STDERR "-???- DEBUG=$DEBUG=\n";
 
 	# OUTTERMOST LOOP TO TRY EACH STREAM-TYPE: (CONSTRAINED IF USER SPECIFIES STREAMTYPES AS EXTRA ARGS TO new())
 	foreach my $streamtype (@okStreams) {
-print STDERR "--OUTTER: type=$streamtype=\n"  if ($DEBUG);
+		print STDERR "--OUTTER: type=$streamtype=\n"  if ($DEBUG);
 		$streamhtml = $streamhtml0;
 		$streampattern = $streamtype;
 		if ($streamtype eq 'secure') {
@@ -331,7 +451,7 @@ print STDERR "--OUTTER: type=$streamtype=\n"  if ($DEBUG);
 		print STDERR "-3: PATTERN=${streampattern}_stream=\n"  if ($DEBUG);
 INNER:  while ($streamhtml =~ s#(${streampattern}_stream)\"\s*\:\s*\"([^\"]+)\"##)
 		{
-print STDERR "----INNER: type=$streampattern=\n"  if ($DEBUG);
+			print STDERR "----INNER: type=$streampattern=\n"  if ($DEBUG);
 			$self->{'streamtype'} = substr($1, 1);
 			$self->{'streamurl'} = $2;
 			foreach my $xp (@skipStreams) {
@@ -399,8 +519,43 @@ sub get
 
 sub getURL   #LIKE GET, BUT ONLY RETURN THE SINGLE ONE W/BEST BANDWIDTH AND RELIABILITY:
 {
-	my $self = $_[0];
-	return ${$self->{'streams'}}[0];
+	my $self = shift;
+	my $arglist = (defined $_[0]) ? join('|',@_) : '';
+	my $idx = ($arglist =~ /\b\-?random\b/) ? int rand scalar @{$self->{'streams'}} : 0;
+	if ($arglist =~ /\b\-?noplaylists\b/ && ${$self->{'streams'}}[$idx] =~ /\.pls$/i) {
+		my $firstStream = ${$self->{'streams'}}[$idx];
+		print STDERR "-getURL($idx): NOPLAYLISTS and (".${$self->{'streams'}}[$idx].")\n"  if ($DEBUG);
+		my $ua = LWP::UserAgent->new(@userAgentOps);		
+		$ua->timeout($uops{'timeout'});
+		$ua->cookie_jar({});
+		$ua->env_proxy;
+		my $html = '';
+		my $response = $ua->get($firstStream);
+		if ($response->is_success) {
+			$html = $response->decoded_content;
+		} else {
+			print STDERR $response->status_line  if ($DEBUG);
+			my $no_wget = system('wget','-V');
+			unless ($no_wget) {
+				print STDERR "\n..trying wget...\n"  if ($DEBUG);
+				$html = `wget -t 2 -T 20 -O- -o /dev/null \"$firstStream\" 2>/dev/null `;
+			}
+		}
+		my @lines = split(/\r?\n/, $html);
+		$firstStream = '';
+		my $firstTitle = '';
+		foreach my $line (@lines) {
+			if ($line =~ m#^\s*File\d+\=(.+)$#) {
+				$firstStream ||= $1;
+			} elsif ($line =~ m#^\s*Title\d+\=(.+)$#) {
+				$firstTitle ||= $1;
+			}
+		}
+		$self->{'title'} ||= $firstTitle;
+		print STDERR "-getURL: first=$firstStream= title=$firstTitle=\n"  if ($DEBUG);
+		return $firstStream || ${$self->{'streams'}}[$idx];
+	}
+	return ${$self->{'streams'}}[$idx];
 }
 
 sub count
@@ -439,8 +594,9 @@ sub getIconData
 	my $self = shift;
 	return ()  unless ($self->{'iconurl'});
 
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+print STDERR "--getIconData: iconurl=".$self->{'iconurl'}."=\n";
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
@@ -453,7 +609,9 @@ sub getIconData
 	return ()  unless ($art_image);
 
 	(my $image_ext = $self->{'iconurl'}) =~ s/^.+\.//;
+print STDERR "--getIconData: ext0=$image_ext=\n";
 	$image_ext =~ s/[^A-Za-z].*$//;
+print STDERR "--getIconData: ext final=$image_ext=\n";
 
 	return ($image_ext, $art_image);
 }
@@ -468,8 +626,8 @@ sub getImageData
 {
 	my $self = shift;
 	return ()  unless ($self->{'imageurl'});
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
