@@ -113,11 +113,18 @@ or just the station ID:  55952.
 
 Returns an array of strings representing all stream urls found.
 
-=item $station->B<getURL>()
+=item $station->B<getURL>([I<options>])
 
 Similar to B<get>() except it only returns a single stream representing 
 the first valid stream found.  
 
+Current options are:  I<-random> and I<-noplaylists>.  By default, the 
+first ("best"?) stream is returned.  If I<-random> is specified, then 
+a random one is selected from the list of streams found.  
+If I<-noplaylists> is specified, and the stream to be returned is a 
+"playlist" (.pls extension), it is first fetched and the first entry 
+in the playlist is returned.  This is needed by Fauxdacious Mediaplayer.
+
 =item $station->B<count>()
 
 Returns the number of streams found for the station.
@@ -154,6 +161,41 @@ Returns the stream's type ("Reciva").
 
 =back
 
+=head1 CONFIGURATION FILES
+
+=over 4
+
+=item ~/.config/StreamFinder/Reciva/config
+
+Optional text file for specifying various configuration options 
+for a specific site (submodule).  Each option is specified on a 
+separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used only by the specific 
+(submodule) specified.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.  
+Blank lines and lines starting with a "#" sign are ignored.
+
+Options specified here override any specified in I<~/.config/StreamFinder/config>.
+
+=item ~/.config/StreamFinder/config
+
+Optional text file for specifying various configuration options.  
+Each option is specified on a separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used by all sites 
+(submodules) that support them.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.
+
+=back
+
+NOTE:  Options specified in the options parameter list will override 
+those corresponding options specified in these files.
+
 =head1 KEYWORDS
 
 reciva
@@ -249,7 +291,9 @@ use warnings;
 use LWP::UserAgent ();
 use vars qw(@ISA @EXPORT);
 
-our $DEBUG = 0;
+my $DEBUG = 0;
+my %uops = ();
+my @userAgentOps = ();
 
 require Exporter;
 
@@ -260,6 +304,32 @@ sub new
 {
 	my $class = shift;
 	my $url = shift;
+
+	my $self = {};
+	return undef  unless ($url);
+
+	foreach my $p ("$ENV{HOME}/.config/StreamFinder/config", "$ENV{HOME}/.config/StreamFinder/Reciva/config") {
+		if (open IN, $p) {
+			my ($atr, $val);
+			while (<IN>) {
+				chomp;
+				next  if (/^\s*\#/o);
+				($atr, $val) = split(/\s*\=\>\s*/o, $_, 2);
+				eval "\$uops{$atr} = $val";
+			}
+			close IN;
+		}
+	}
+	foreach my $i (qw(agent from conn_cache default_headers local_address ssl_opts max_size
+			max_redirect parse_head protocols_allowed protocols_forbidden requests_redirectable
+			proxy no_proxy)) {
+		push @userAgentOps, $i, $uops{$i}  if (defined $uops{$i});
+	}
+	push (@userAgentOps, 'agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0')
+			unless (defined $uops{'agent'});
+	$uops{'timeout'} = 10  unless (defined $uops{'timeout'});
+	$DEBUG = $uops{'debug'}  if (defined $uops{'debug'});
+
 	while (@_) {
 		if ($_[0] =~ /^\-?debug$/o) {
 			shift;
@@ -267,16 +337,13 @@ sub new
 		}
 	}	
 
-	my $self = {};
-	return undef  unless ($url);
-
 	$url = 'https://radios.reciva.com/station/' . $url  unless ($url =~ /^http/);
 	(my $url2fetch = $url) =~ s#station\/(\d+).*$#streamer\?stationid\=$1\&streamnumber=0#;
 	$self->{'id'} = $1;
 	my $html = '';
 	print STDERR "-0(Reciva): URL=$url2fetch=\n"  if ($DEBUG);
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $response = $ua->get($url);
@@ -284,8 +351,11 @@ sub new
 		$html = $response->decoded_content;
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
-		print STDERR "\n..trying wget...\n"  if ($DEBUG);
-		$html = `wget -t 2 -T 20 -O- -o /dev/null \"$url\" 2>/dev/null `;
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			$html = `wget -t 2 -T 20 -O- -o /dev/null \"$url\" 2>/dev/null `;
+		}
 	}
 	print STDERR "-1: html=$html=\n"  if ($DEBUG > 1);
 	return undef  unless ($html);  #STEP 1 FAILED, INVALID STATION URL, PUNT!
@@ -377,12 +447,43 @@ sub get
 
 sub getURL   #LIKE GET, BUT ONLY RANDOMLY SELECT ONE TO RETURN:
 {
-	my $self = $_[0];
-	my $streamNumber = int rand scalar @{$self->{'streams'}};
-	$streamNumber = $#{$self->{'streams'}}  if ($streamNumber > $#{$self->{'streams'}});
-	$streamNumber = scalar(@{$self->{'streams'}}) + $streamNumber  if ($streamNumber < 0);
-	$streamNumber = 0  if ($streamNumber < 0);
-	return ${$self->{'streams'}}[$streamNumber];  #URL TO RANDOM PLAYABLE STREAM.
+	my $self = shift;
+	my $arglist = (defined $_[0]) ? join('|',@_) : '';
+	my $idx = ($arglist =~ /\b\-?random\b/) ? int rand scalar @{$self->{'streams'}} : 0;
+	if ($arglist =~ /\b\-?noplaylists\b/ && ${$self->{'streams'}}[$idx] =~ /\.pls$/i) {
+		my $firstStream = ${$self->{'streams'}}[$idx];
+		print STDERR "-getURL($idx): NOPLAYLISTS and (".${$self->{'streams'}}[$idx].")\n"  if ($DEBUG);
+		my $ua = LWP::UserAgent->new(@userAgentOps);		
+		$ua->timeout($uops{'timeout'});
+		$ua->cookie_jar({});
+		$ua->env_proxy;
+		my $html = '';
+		my $response = $ua->get($firstStream);
+		if ($response->is_success) {
+			$html = $response->decoded_content;
+		} else {
+			print STDERR $response->status_line  if ($DEBUG);
+			my $no_wget = system('wget','-V');
+			unless ($no_wget) {
+				print STDERR "\n..trying wget...\n"  if ($DEBUG);
+				$html = `wget -t 2 -T 20 -O- -o /dev/null \"$firstStream\" 2>/dev/null `;
+			}
+		}
+		my @lines = split(/\r?\n/, $html);
+		$firstStream = '';
+		my $firstTitle = '';
+		foreach my $line (@lines) {
+			if ($line =~ m#^\s*File\d+\=(.+)$#) {
+				$firstStream ||= $1;
+			} elsif ($line =~ m#^\s*Title\d+\=(.+)$#) {
+				$firstTitle ||= $1;
+			}
+		}
+		$self->{'title'} ||= $firstTitle;
+		print STDERR "-getURL: first=$firstStream= title=$firstTitle=\n"  if ($DEBUG);
+		return $firstStream || ${$self->{'streams'}}[$idx];
+	}
+	return ${$self->{'streams'}}[$idx];
 }
 
 sub count
@@ -419,8 +520,8 @@ sub getIconData
 {
 	my $self = shift;
 	return ()  unless ($self->{'iconurl'});
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
@@ -428,10 +529,13 @@ sub getIconData
 	if ($response->is_success) {
 		$art_image = $response->decoded_content;
 	} else {
-		my $iconurl = $self->{'iconurl'};
 		print STDERR $response->status_line  if ($DEBUG);
-		print STDERR "\n..trying wget...\n"  if ($DEBUG);
-		$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconurl\" 2>/dev/null `;
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			my $iconUrl = $self->{'iconurl'};
+			$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconUrl\" 2>/dev/null `;
+		}
 	}
 	return ()  unless ($art_image);
 	(my $image_ext = $self->{'iconurl'}) =~ s/^.+\.//;
@@ -449,8 +553,8 @@ sub getImageData
 {
 	my $self = shift;
 	return ()  unless ($self->{'imageurl'});
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
@@ -458,10 +562,13 @@ sub getImageData
 	if ($response->is_success) {
 		$art_image = $response->decoded_content;
 	} else {
-		my $imgurl = $self->{'imageurl'};
 		print STDERR $response->status_line  if ($DEBUG);
-		print STDERR "\n..trying wget...\n"  if ($DEBUG);
-		$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$imgurl\" 2>/dev/null `;
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			my $iconUrl = $self->{'iconurl'};
+			$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconUrl\" 2>/dev/null `;
+		}
 	}
 	return ()  unless ($art_image);
 	my $image_ext = $self->{'imageurl'};
